import random
import string
import uuid

from datetime import datetime
from typing import TYPE_CHECKING, List

from barker.models.login_history import LoginHistory
from sqlalchemy import Boolean, DateTime, ForeignKey, Unicode, desc, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, Session, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .section import Section


@reg.mapped_as_dataclass(unsafe_hash=True)
class Device:
    __tablename__ = "devices"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    name: Mapped[str] = mapped_column("name", Unicode(255), unique=True, nullable=False)
    enabled: Mapped[bool] = mapped_column("enabled", Boolean, nullable=False)
    section_id: Mapped[uuid.UUID] = mapped_column(
        "section_id", UUID(as_uuid=True), ForeignKey("sections.id"), nullable=False
    )
    creation_date: Mapped[datetime] = mapped_column("creation_date", DateTime(), nullable=False)

    section: Mapped["Section"] = relationship(back_populates="devices")
    login_history: Mapped[List["LoginHistory"]] = relationship(
        order_by=desc(LoginHistory.date), back_populates="device"
    )

    def __init__(self, name=None, enabled=None, section_id=None, creation_date=None, id_=None):
        self.name = name
        self.enabled = enabled
        self.section_id = section_id
        self.creation_date = datetime.utcnow() if creation_date is None else creation_date
        self.id = id_

    @classmethod
    def create(cls, db: Session):
        main_section = uuid.UUID("3f13f6e7-dc76-4fca-8fdb-b2bbf29b35df")
        name = "".join(random.choices(string.ascii_lowercase + string.digits, k=6))
        device = Device(name, False, main_section)
        db.add(device)
        return device
