import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { CustomerDetailComponent } from './customer-detail/customer-detail.component';
import { CustomerListComponent } from './customer-list/customer-list.component';
import { CustomerListResolver } from './customer-list-resolver.service';
import { CustomerResolver } from './customer-resolver.service';

const customersRoutes: Routes = [
  {
    path: '',
    component: CustomerListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Customers',
    },
    resolve: {
      list: CustomerListResolver,
    },
  },
  {
    path: 'new',
    component: CustomerDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Customers',
    },
    resolve: {
      item: CustomerResolver,
    },
  },
  {
    path: ':id',
    component: CustomerDetailComponent,
    canActivate: [AuthGuard],
    resolve: {
      item: CustomerResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(customersRoutes)],
  exports: [RouterModule],
  providers: [CustomerListResolver, CustomerResolver],
})
export class CustomerRoutingModule {}
