import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { catchError } from 'rxjs/operators';

import { Device } from '../core/device';
import { ErrorLoggerService } from '../core/error-logger.service';

const url = '/api/devices';
const serviceName = 'DeviceService';

@Injectable({
  providedIn: 'root',
})
export class DeviceService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Device> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return this.http
      .get<Device>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Device>;
  }

  list(): Observable<Device[]> {
    return this.http
      .get<Device[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Device[]>;
  }

  save(device: Device): Observable<Device> {
    return this.http
      .post<Device>(`${url}`, device)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Device>;
  }

  update(device: Device): Observable<Device> {
    return this.http
      .put<Device>(`${url}/${device.id}`, device)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Device>;
  }

  saveOrUpdate(device: Device): Observable<Device> {
    if (!device.id) {
      return this.save(device);
    }
    return this.update(device);
  }

  delete(id: string): Observable<Device> {
    return this.http
      .delete<Device>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Device>;
  }
}
