import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { TaxListResolver } from '../taxes/tax-list-resolver.service';

import { ProductListResolver } from './product-list-resolver.service';
import { SaleCategoryDetailComponent } from './sale-category-detail/sale-category-detail.component';
import { SaleCategoryListComponent } from './sale-category-list/sale-category-list.component';
import { SaleCategoryListResolver } from './sale-category-list-resolver.service';
import { SaleCategoryResolver } from './sale-category-resolver.service';

const saleCategoriesRoutes: Routes = [
  {
    path: '',
    component: SaleCategoryListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      list: SaleCategoryListResolver,
    },
  },
  {
    path: 'new',
    component: SaleCategoryDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: SaleCategoryResolver,
      taxes: TaxListResolver,
      products: ProductListResolver,
    },
  },
  {
    path: ':id',
    component: SaleCategoryDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: SaleCategoryResolver,
      taxes: TaxListResolver,
      products: ProductListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(saleCategoriesRoutes)],
  exports: [RouterModule],
  providers: [SaleCategoryListResolver, SaleCategoryResolver],
})
export class SaleCategoriesRoutingModule {}
